@@ -25,10 +25,10 @@ htmlFile <- function(filename, svgdev) {
   fn <- saveXML(obj, file = htmlfile)
 }
 
-svgOpen <- function(width=200, height=200) {
+svgOpen <- function(width=200, height=200, strict=TRUE, rootAttrs=NULL) {
   # Ensure all vp contexts are now zero
   assign("contextLevels", 0, envir = .gridSVGEnv)
-  svgdev <- svgDevice(width, height)
+  svgdev <- svgDevice(width, height, strict, rootAttrs)
   svgHeader(width, height, svgdev)
   return(svgdev)
 }
@@ -299,11 +299,11 @@ svgStartGroup <- function(id=NULL, clip=FALSE, mask=FALSE,
   if (has.link)
     svgStartLink(links[id], show[id], svgdev)
 
-  attrlist <- list(id = prefixName(id),
-                   svgClipAttr(id, clip),
-                   svgMaskAttr(id, mask),
-                   svgStyleAttributes(style),
-                   svgAttribTxt(attributes, id))
+  attrlist <- c(list(id = prefixName(id)),
+                svgClipAttr(id, clip),
+                svgMaskAttr(id, mask),
+                svgStyleAttributes(style, svgdev),
+                svgAttribTxt(attributes, id, "g", svgdev))
   attrlist <- attrList(attrlist)
 
   # Avoid clobbering "class" attribute if it exists
@@ -574,11 +574,6 @@ svgAnimateScale <- function(xvalues, yvalues,
 svgLines <- function(x, y, id=NULL, arrow = NULL,
                      attributes=svgAttrib(), links=NULL, show=NULL,
                      style=svgStyle(), svgdev=svgDevice()) {
-  # Grabbing arrow info for marker element references
-  if (! is.null(arrow$ends))
-      lineMarkerTxt <- markerTxt(arrow$ends, id)
-  else
-      lineMarkerTxt <- NULL
 
   # Never fill a line
   style$fill <- "none"
@@ -587,17 +582,47 @@ svgLines <- function(x, y, id=NULL, arrow = NULL,
   if (has.link)
     svgStartLink(links[id], show[id], svgdev)
 
-  attrlist <- list(id = prefixName(id),
-                   points = paste0(round(x, 2), ",",
-                                   round(y, 2),
-                                   collapse=" "),
-                   lineMarkerTxt,
-                   svgStyleAttributes(style),
-                   svgAttribTxt(attributes, id))
-  attrlist <- attrList(attrlist)
-  newXMLNode("polyline", parent = svgDevParent(svgdev),
-             attrs = attrlist)
-
+  # Handle NA values in (x, y)
+  xylist <- splitOnNA(x, y)
+  N <- length(xylist)
+  if (N > 0) {
+      if (N > 1) {
+          # assume no more than 26 NAs in (x, y)!
+          alpha <- letters[1:N]
+      } else {
+          alpha <- ""
+      }
+      for (i in 1:N) {
+          # Grabbing arrow info for marker element references
+          # Arrows NOT drawn at NA splits
+          if (! is.null(arrow$ends)) {
+              if (arrow$ends == "both" && i == 1 && N == 1) 
+                  lineMarkerTxt <- markerTxt("both", id)
+              else if ((arrow$ends == "both" || arrow$ends == "first") &&
+                       i == 1 && is.finite(x[1]) && is.finite(y[1]))
+                  lineMarkerTxt <- markerTxt("first", id)
+              else if ((arrow$ends == "both" || arrow$ends == "last") &&
+                       i == N &&
+                       is.finite(x[length(x)]) && is.finite(y[length(y)]))
+                  lineMarkerTxt <- markerTxt("last", id)
+              else 
+                  lineMarkerTxt <- NULL
+          } else {
+              lineMarkerTxt <- NULL
+          }
+          attrlist <- c(list(id = prefixName(paste0(id, alpha[i])),
+                             points = paste0(round(xylist[[i]]$x, 2), ",",
+                                 round(xylist[[i]]$y, 2),
+                                 collapse=" ")),
+                        lineMarkerTxt,
+                        svgStyleAttributes(style, svgdev),
+                        svgAttribTxt(attributes, id, "polyline", svgdev))
+          attrlist <- attrList(attrlist)
+          newXMLNode("polyline", parent = svgDevParent(svgdev),
+                     attrs = attrlist)
+      }
+  }
+  
   if (has.link)
     svgEndLink(svgdev)
 }
@@ -641,14 +666,13 @@ svgMarker <- function(x, y, type, ends, direction, name,
     ids <- markerName("both", name)
     refXs <- direction * round(c(-width, width), 2)
     refYs <- round(c(-height / 2, height / 2), 2)
-    pathlist <- attrList(list(d = d, svgStyleAttributes(style)))
+    pathlist <- attrList(c(list(d = d), svgStyleAttributes(style, svgdev)))
     # It is possible for width to be 0, i.e. when angle=90.
     # Ensure that the marker is always at least as wide as the
     # stroke width that it is given.
     mwidth <- max(as.numeric(pathlist$`stroke-width`), width)
     mheight <- max(as.numeric(pathlist$`stroke-width`), height)
-    pathattrs <- list(pathlist,
-                      pathlist)
+    pathattrs <- list(pathlist, pathlist)
     pathattrs[[1]]$transform <- "rotate(180)"
 
     newXMLNode("defs", parent = svgDevParent(svgdev),
@@ -708,15 +732,28 @@ svgPolygon <- function(x, y, id=NULL,
   if (has.link)
     svgStartLink(links[id], show[id], svgdev)
 
-  tmpattr <- list(id = prefixName(id),
-                  points = paste0(round(x, 2), ",",
-                                  round(y, 2),
-                                  collapse = " "),
-                  svgStyleAttributes(style),
-                  svgAttribTxt(attributes, id))
-  tmpattr <- attrList(tmpattr)
-  newXMLNode("polygon", parent = svgDevParent(svgdev),
-             attrs = tmpattr)
+  # Handle NA values in (x, y)
+  xylist <- splitOnNA(x, y)
+  N <- length(xylist)
+  if (N > 0) {
+      if (N > 1) {
+          # assume no more than 26 NAs in (x, y)!
+          alpha <- letters[1:N]
+      } else {
+          alpha <- ""
+      }
+      for (i in 1:N) {
+          attrlist <- c(list(id = prefixName(paste0(id, alpha[i])),
+                             points = paste0(round(xylist[[i]]$x, 2), ",",
+                                 round(xylist[[i]]$y, 2),
+                                 collapse=" ")),
+                        svgStyleAttributes(style, svgdev),
+                        svgAttribTxt(attributes, id, "polygon", svgdev))
+          attrlist <- attrList(attrlist)
+          newXMLNode("polygon", parent = svgDevParent(svgdev),
+                     attrs = attrlist)
+      }
+  }
 
   if (has.link)
     svgEndLink(svgdev)
@@ -730,11 +767,19 @@ svgPath <- function(x, y, rule, id=NULL,
         stop("x and y must be same length")
     if (is.atomic(x)) {
         if (is.atomic(y)) {
-            x <- list(x)
-            y <- list(y)
+            # NAs allowed;  turn into sub-paths
+            xy <- splitOnNA(x, y)
+            x <- lapply(xy, "[[", "x")
+            y <- lapply(xy, "[[", "y")
         } else {
             stop("'x' and 'y' must both be lists or both be atomic")
         }
+        # If the path contains multiple sub-paths then NAs are NOT allowed
+        # (to follow 'grid' behaviour)
+        bad <- !all(sapply(x, function(z) { all(is.finite(z)) })) ||
+            !all(sapply(y, function(z) { all(is.finite(z)) }))
+        if (bad)
+            stop("non-finite x or y in graphics path")
     }
     n <- length(x)
     d <- mapply(function(subx, suby) {
@@ -745,11 +790,11 @@ svgPath <- function(x, y, rule, id=NULL,
                           "Z")
                 }, x, y)
 
-    tmpattr <- list(id = prefixName(id),
-                    d = paste(unlist(d), collapse = " "),
-                    "fill-rule" = switch(rule, winding="nonzero", "evenodd"),
-                    svgStyleAttributes(style),
-                    svgAttribTxt(attributes, id))
+    tmpattr <- c(list(id = prefixName(id),
+                      d = paste(unlist(d), collapse = " "),
+                      "fill-rule" = switch(rule, winding="nonzero", "evenodd")),
+                 svgStyleAttributes(style, svgdev),
+                 svgAttribTxt(attributes, id, "path", svgdev))
     tmpattr <- attrList(tmpattr)
 
 
@@ -768,6 +813,10 @@ svgRaster <- function(x, y, width, height, angle=0, datauri, id=NULL,
                       just, vjust, hjust,
                       attributes=svgAttrib(), links=NULL, show=NULL,
                       style=svgStyle(), svgdev=svgDevice()) {
+    # Draw nothing if non-finite location or size
+    if (!(is.finite(x) && is.finite(y) &&
+              is.finite(width) && is.finite(height)))
+        return()
     has.link <- hasLink(links[id])
     if (has.link)
         svgStartLink(links[id], show[id], svgdev)
@@ -789,10 +838,10 @@ svgRaster <- function(x, y, width, height, angle=0, datauri, id=NULL,
     if (!is.null(angleTransform)) {
         transform <- paste(angleTransform, transform)
     }
-    attrlist <- list(id = prefixName(id),
-                     transform = transform,
-                     svgStyleAttributes(style),
-                     svgAttribTxt(attributes, id))
+    attrlist <- c(list(id = prefixName(id),
+                       transform = transform),
+                  svgStyleAttributes(style, svgdev),
+                  svgAttribTxt(attributes, id, "g", svgdev))
     attrlist <- attrList(attrlist)
     newXMLNode("g", parent = svgDevParent(svgdev),
                attrs = attrlist,
@@ -821,37 +870,41 @@ svgRaster <- function(x, y, width, height, angle=0, datauri, id=NULL,
 svgRect <- function(x, y, width, height, angle=0, id=NULL,
                     attributes=svgAttrib(), links=NULL, show=NULL,
                     style=svgStyle(), svgdev=svgDevice()) {
-  has.link <- hasLink(links[id])
-  if (has.link)
-    svgStartLink(links[id], show[id], svgdev)
-
-  if (width < 0) {
-    x <- x + width # shifts x to the left
-    width <- abs(width)
-  }
-
-  if (height < 0) {
-    y <- y + height # shifts y down
-    height <- abs(height)
-  }
-  
-  rx <- round(x, 2)
-  ry <- round(y, 2)
-
-  attrlist <- list(id = prefixName(id),
-                   x = rx,
-                   y = ry,
-                   width = round(width, 2),
-                   height = round(height, 2),
-                   transform = svgAngleTransform(rx, ry, angle), 
-                   svgStyleAttributes(style),
-                   svgAttribTxt(attributes, id))
-  attrlist <- attrList(attrlist)
-  newXMLNode("rect", parent = svgDevParent(svgdev),
-             attrs = attrlist)
-
-  if (has.link)
-    svgEndLink(svgdev)
+    # Draw nothing if non-finite location or size
+    if (!(is.finite(x) && is.finite(y) &&
+              is.finite(width) && is.finite(height)))
+        return()
+    has.link <- hasLink(links[id])
+    if (has.link)
+        svgStartLink(links[id], show[id], svgdev)
+    
+    if (width < 0) {
+        x <- x + width # shifts x to the left
+        width <- abs(width)
+    }
+    
+    if (height < 0) {
+        y <- y + height # shifts y down
+        height <- abs(height)
+    }
+    
+    rx <- round(x, 2)
+    ry <- round(y, 2)
+    
+    attrlist <- c(list(id = prefixName(id),
+                       x = rx,
+                       y = ry,
+                       width = round(width, 2),
+                       height = round(height, 2),
+                       transform = svgAngleTransform(rx, ry, angle)), 
+                  svgStyleAttributes(style, svgdev),
+                  svgAttribTxt(attributes, id, "rect", svgdev))
+    attrlist <- attrList(attrlist)
+    newXMLNode("rect", parent = svgDevParent(svgdev),
+               attrs = attrlist)
+    
+    if (has.link)
+        svgEndLink(svgdev)
 }
 
 svgTextSplitLines <- function(text, id, lineheight, charheight,
@@ -896,12 +949,12 @@ svgTextElement <- function(text, id, rot, hjust, vjust,
                    list(transform = paste0("rotate(", round(-rot, 2), ")"))
                  else
                    NULL
-    attrlist <- list(x = 0,
-                     y = 0,
-                     id = paste(id, "text", sep=getSVGoption("id.sep")),
-                     transform,
-                     textAnchor(hjust),
-                     svgStyleAttributes(style))
+    attrlist <- c(list(x = 0,
+                       y = 0,
+                       id = paste(id, "text", sep=getSVGoption("id.sep"))),
+                  transform,
+                  textAnchor(hjust),
+                  svgStyleAttributes(style, svgdev))
     attrlist <- attrList(attrlist)
     newpar <- newXMLNode("text", parent = svgDevParent(svgdev),
                          attrs = attrlist)
@@ -949,12 +1002,12 @@ svgMathElement <- function(text, id, rot, hjust, vjust,
             y <- (ascent - fontheight)
     }
 
-    tmpattr <- list(x = round(x, 2),
-                    y = round(y, 2),
-                    id = paste(id, "mathtext", sep=getSVGoption("id.sep")),
-                    width = round(3*width, 2),
-                    height = round(3*height, 2),
-                    svgStyleAttributes(style))
+    tmpattr <- c(list(x = round(x, 2),
+                      y = round(y, 2),
+                      id = paste(id, "mathtext", sep=getSVGoption("id.sep")),
+                      width = round(3*width, 2),
+                      height = round(3*height, 2)),
+                 svgStyleAttributes(style, svgdev))
     if (rot != 0)
         tmpattr$transform <- paste0("rotate(", round(-rot, 2), ")")
 
@@ -972,6 +1025,9 @@ svgText <- function(x, y, text, hjust="left", vjust="bottom", rot=0,
                     fontfamily="sans", fontface="plain",
                     id=NULL, attributes=svgAttrib(), links=NULL, show=NULL,
                     style=svgStyle(), svgdev=svgDevice()) {
+    # Draw nothing if x/y non-finite
+    if (!(is.finite(x) && is.finite(y)))
+        return()
     has.link <- hasLink(links[id])
     if (has.link)
         svgStartLink(links[id], show[id], svgdev)
@@ -996,7 +1052,7 @@ svgText <- function(x, y, text, hjust="left", vjust="bottom", rot=0,
         topattrs$transform <- paste(angleTransform, topattrs$transform)
     }
     topattrs$`stroke-width` <- "0.1"
-    topattrs <- c(topattrs, svgAttribTxt(attributes, id))
+    topattrs <- c(topattrs, svgAttribTxt(attributes, id, "g", svgdev))
 
     # Flip the y-direction again so that text is drawn "upright"
     # Do the flip in a separate <g> so that can animate the
@@ -1035,23 +1091,26 @@ svgText <- function(x, y, text, hjust="left", vjust="bottom", rot=0,
 svgCircle <- function(x, y, r, id=NULL,
                       attributes=svgAttrib(), links=NULL, show=NULL,
                       style=svgStyle(), svgdev=svgDevice()) {
-  has.link <- hasLink(links[id])
-  if (has.link)
-    svgStartLink(links[id], show[id], svgdev)
-
-  tmpattr <- list(id = prefixName(id),
-                  cx = round(x, 2),
-                  cy = round(y, 2),
-                  r = round(r, 2),
-                  svgStyleAttributes(style),
-                  svgAttribTxt(attributes, id))
-  tmpattr <- attrList(tmpattr)
-  has.link <- hasLink(links[id])
-  newXMLNode("circle", parent = svgDevParent(svgdev),
+    # Draw nothing if non-finite location or size
+    if (!(is.finite(x) && is.finite(y) && is.finite(r)))
+        return()
+    has.link <- hasLink(links[id])
+    if (has.link)
+        svgStartLink(links[id], show[id], svgdev)
+    
+    tmpattr <- c(list(id = prefixName(id),
+                      cx = round(x, 2),
+                      cy = round(y, 2),
+                      r = round(r, 2)),
+                 svgStyleAttributes(style, svgdev),
+                 svgAttribTxt(attributes, id, "circle", svgdev))
+    tmpattr <- attrList(tmpattr)
+    has.link <- hasLink(links[id])
+    newXMLNode("circle", parent = svgDevParent(svgdev),
              attrs = tmpattr)
-
-  if (has.link)
-    svgEndLink(svgdev)
+    
+    if (has.link)
+        svgEndLink(svgdev)
 }
 
 svgScript <- function(body, href, type="application/ecmascript",
@@ -1083,76 +1142,79 @@ svgUseSymbol <- function(id, x, y, size, pch, angle=0,
                          attributes=svgAttrib(), links=NULL, show=NULL,
                          style=svgStyle(), svgdev=svgDevice()) {
 
-  has.link <- hasLink(links[id])
-  if (has.link)
-    svgStartLink(links[id], show[id], svgdev)
+    # Draw nothing if non-finite location or size
+    if (!(is.finite(x) && is.finite(y) && is.finite(size)))
+        return()
+    has.link <- hasLink(links[id])
+    if (has.link)
+        svgStartLink(links[id], show[id], svgdev)
 
-  # Ensure the "dot" is only 1px wide
-  if (pch == ".")
-    size <- 1
+    # Ensure the "dot" is only 1px wide
+    if (pch == ".")
+        size <- 1
 
-  # Ensure we refer to the correct <symbol> id
-  numpch <- if (is.character(pch))
-                as.numeric(charToRaw(pch))
-            else
-                pch
+    # Ensure we refer to the correct <symbol> id
+    numpch <- if (is.character(pch))
+                  as.numeric(charToRaw(pch))
+              else
+                  pch
+    
+    rx <- round(x, 2)
+    ry <- round(y, 2)
+    
+    tmpattr <- list(id = prefixName(id),
+                    "xlink:href" =
+                        paste0("#", prefixName(paste0("gridSVG.pch", numpch))),
+                    x = rx, y = ry,
+                    width = round(size, 2),
+                    height = round(size, 2))
 
-  rx <- round(x, 2)
-  ry <- round(y, 2)
-  
-  tmpattr <- list(id = prefixName(id),
-                  "xlink:href" =
-                    paste0("#", prefixName(paste0("gridSVG.pch", numpch))),
-                  x = rx, y = ry,
-                  width = round(size, 2),
-                  height = round(size, 2))
-
-  # centering adjustment
-  r <- round(-size / 2, 2)
-  tmpattr$transform <- paste0("translate(", r, ",", r, ")")
-  angleTransform <- svgAngleTransform(rx, ry, angle)
-  if (!is.null(angleTransform)) {
-      tmpattr$transform <- paste(angleTransform, tmpattr$transform)
-  }
-  
-  # Preserve order
-  tmpattr <- c(tmpattr,
-               svgStyleAttributes(style),
-               svgAttribTxt(attributes, id))
-
-  # Need to scale the stroke width otherwise for large points
-  # we also have large strokes
-  sw <- as.numeric(tmpattr$`stroke-width`)
-  scalef <- size / 10 # 10 is the point viewBox size
-  sw <- sw / scalef
-  tmpattr$`stroke-width` <- round(sw, 2)
-
-  # For pch outside 0-25 or characters
-  if (is.character(pch) || (is.numeric(pch) && pch > 25)) {
-    # When we have a "." we have a special case
-    if ((is.character(pch) && pch == ".") ||
-        (is.numeric(pch) && pch == 46)) {
-      # Strip unnecessary attribs
-      fsind <- which(names(tmpattr) == "font-size")
-      if (length(fsind) > 0)
-        tmpattr <- tmpattr[-fsind]
-      # Because we really want just a dot, use crispEdges
-      # as anti-aliasing isn't really necessary
-      tmpattr$`shape-rendering` <- "crispEdges"
-    } else {
-      # Make the s-w small so we see a stroke just barely
-      tmpattr$`stroke-width` <- "0.1"
-      # Set the font-size, otherwise it's going to mess with our scaling.
-      # 10px so it's the size of the point definition
-      tmpattr$`font-size` <- "10"
+    # centering adjustment
+    r <- round(-size / 2, 2)
+    tmpattr$transform <- paste0("translate(", r, ",", r, ")")
+    angleTransform <- svgAngleTransform(rx, ry, angle)
+    if (!is.null(angleTransform)) {
+        tmpattr$transform <- paste(angleTransform, tmpattr$transform)
     }
-  }
+  
+    # Preserve order
+    tmpattr <- c(tmpattr,
+                 svgStyleAttributes(style, svgdev),
+                 svgAttribTxt(attributes, id, "use", svgdev))
+    
+    # Need to scale the stroke width otherwise for large points
+    # we also have large strokes
+    sw <- as.numeric(tmpattr$`stroke-width`)
+    scalef <- size / 10 # 10 is the point viewBox size
+    sw <- sw / scalef
+    tmpattr$`stroke-width` <- round(sw, 2)
 
-  newXMLNode("use", parent = svgDevParent(svgdev),
-             attrs = attrList(tmpattr))
+    # For pch outside 0-25 or characters
+    if (is.character(pch) || (is.numeric(pch) && pch > 25)) {
+        # When we have a "." we have a special case
+        if ((is.character(pch) && pch == ".") ||
+            (is.numeric(pch) && pch == 46)) {
+            # Strip unnecessary attribs
+            fsind <- which(names(tmpattr) == "font-size")
+            if (length(fsind) > 0)
+                tmpattr <- tmpattr[-fsind]
+            # Because we really want just a dot, use crispEdges
+            # as anti-aliasing isn't really necessary
+            tmpattr$`shape-rendering` <- "crispEdges"
+        } else {
+            # Make the s-w small so we see a stroke just barely
+            tmpattr$`stroke-width` <- "0.1"
+            # Set the font-size, otherwise it's going to mess with our scaling.
+            # 10px so it's the size of the point definition
+            tmpattr$`font-size` <- "10"
+        }
+    }
+    
+    newXMLNode("use", parent = svgDevParent(svgdev),
+               attrs = attrList(tmpattr))
 
-  if (has.link)
-    svgEndLink(svgdev)
+    if (has.link)
+        svgEndLink(svgdev)
 }
 
 # Dispatching function, simply following a naming scheme,
@@ -1460,11 +1522,13 @@ svgPoint25 <- function(svgdev = svgDevice()) {
 # to be defined within user coordinates (see svgPushViewport
 # and svgPopViewport)
 
-svgDevice <- function(width=200, height=200) {
+svgDevice <- function(width=200, height=200, strict=TRUE, rootAttrs) {
   dev <- new.env(FALSE, emptyenv())
   assign("width", width, envir=dev)
   assign("height", height, envir=dev)
   assign("parent", NULL, envir=dev)
+  assign("strict", strict, envir=dev)
+  assign("rootAttrs", rootAttrs, envir=dev)
   assign("id", 1, envir=dev)
   return(dev)
 }
@@ -1477,6 +1541,14 @@ svgDevHeight <- function(svgdev) {
   get("height", envir=svgdev)
 }
 
+svgStrict <- function(svgdev) {
+    get("strict", envir=svgdev)
+}
+
+svgRootAttrs <- function(svgdev) {
+    get("rootAttrs", envir=svgdev)
+}
+
 svgDevParent <- function(svgdev) {
   get("parent", envir=svgdev)
 }
@@ -1511,25 +1583,43 @@ incID <- function(svgdev, n=1) {
 svgHeader <- function(width, height, svgdev=svgDevice()) {
     # This header tested on standalone SVG file in Firefox 3
     # FIXME:  add default xmlns for animation and scripts too?
-    attrs <- list(width = paste0(round(width, 2), "px"),
-                  height = paste0(round(height, 2), "px"),
-                  viewBox = paste(0, 0, round(width, 2), round(height, 2)),
-                  version = "1.1")
-    # Give the <svg> element an ID only if there is a prefix
-    if (nzchar(get("prefix", envir = .gridSVGEnv)))
-        attrs <- c(list(id = get("prefix", envir = .gridSVGEnv)), attrs)
+    attrs <- svgRootAttrs(svgdev)
+    # Resolve any conflict between rootAttrs[c("width", "height")]
+    # and 'width' and 'height' passed in (from device size)
+    if (!"width" %in% names(attrs)) {
+        attrs$width <- paste0(round(width, 2), "px")
+    }
+    if (!"height" %in% names(attrs)) {
+        attrs$height <- paste0(round(height, 2), "px")
+    }
+    if (!"viewBox" %in% names(attrs)) {
+        attrs$viewBox <- paste(0, 0, round(width, 2), round(height, 2))
+    }
+    if (!"version" %in% names(attrs)) {
+        attrs$version <- "1.1"
+    }
+    # Give the <svg> element an ID only if there is a prefix AND
+    # 'rootAttrs' does not specify an ID
+    if (!"id" %in% names(attrs)) {
+        if (nzchar(get("prefix", envir = .gridSVGEnv))) {
+            attrs$id <- get("prefix", envir = .gridSVGEnv)
+        }
+    }
+    if (svgStrict(svgdev)) {
+        checkAttrs(attrs, "svg")
+    }
     svgdoc <-
         newXMLDoc(namespaces = list("http://www.w3.org/2000/svg",
-                       xlink = "http://www.w3.org/1999/xlink"), node = 
-        newXMLNode("svg", attrs = attrs,
-                   namespaceDefinitions = list("http://www.w3.org/2000/svg",
-                       xlink = "http://www.w3.org/1999/xlink")))
+                      xlink = "http://www.w3.org/1999/xlink"),
+                  node =  newXMLNode("svg", attrs = attrList(attrs),
+                      namespaceDefinitions = list("http://www.w3.org/2000/svg",
+                          xlink = "http://www.w3.org/1999/xlink")))
     # Invert the y-axis so that y and height values measure "up"
     rootg <- newXMLNode("g",
                         parent = xmlRoot(svgdoc),
                         attrs = list(transform = paste0("translate(0, ",
-                                                        round(svgDevHeight(svgdev), 2),
-                                                        ") scale(1, -1)")))
+                                         round(svgDevHeight(svgdev), 2),
+                                         ") scale(1, -1)")))
     svgDevChangeParent(rootg, svgdev)
 }
 
@@ -1547,8 +1637,10 @@ svgAttrib <- function(...) {
 # Removes NULL values and flattens our attrib list
 # so we can include lists as elements in "alist"
 # and arrive at a flattened list
+# ALSO removes names from attr values so they don't corrupt attr names
+# e.g., avoid list(a=c(b=1)) becoming c(a.b=1)
 attrList <- function(alist) {
-  as.list(unlist(alist))
+  as.list(unlist(lapply(alist, unname)))
 }
 
 listToSVGAttrib <- function(alist) {
@@ -1566,8 +1658,22 @@ svgClassList <- function(classes) {
         list(class = paste0(unique(classes), collapse = " "))
 }
 
+checkAttrs <- function(attrList, eltName) {
+    attrTable <- get("genAttrTable", .gridSVGEnv)
+    validAttrs <- attrTable$attribute[attrTable$element == eltName]
+    names <- names(attrList)
+    svgnames <- names %in% validAttrs
+    if (!all(svgnames)) {
+        warning(paste("Removing non-SVG attribute name(s):",
+                      paste(names[!svgnames], collapse=", ")))
+        attrList[names[svgnames]]
+    } else {
+        attrList
+    }
+}
+
 # Only use the attributes that are for this 'id'
-svgAttribTxt <- function(attributes, id) {
+svgAttribTxt <- function(attributes, id, eltName, svgdev) {
     if (emptyAttrib(attributes)) {
         list()
     } else {
@@ -1582,16 +1688,22 @@ svgAttribTxt <- function(attributes, id) {
                              id)
         # Drop NULL attributes
         attributes <- attributes[!sapply(attributes, is.null)]
-
         # Need to wipe out names because it messes things up when we
         # need to create an attribute list for nodes
-        if (length(attributes) > 0)
-            lapply(attributes, function(x) {
-                       names(x) <- NULL
-                       x
-                   })
-        else
+        if (length(attributes) > 0) {
+            attrs <- lapply(attributes,
+                            function(x) {
+                                names(x) <- NULL
+                                x
+                            })
+            if (svgStrict(svgdev)) {
+                checkAttrs(attrs, eltName)
+            } else {
+                attrs
+            }
+        } else {
             list()
+        }
     }
 }
 
@@ -1626,29 +1738,6 @@ svgStyleCSS <- function(svgstyle) {
 }
 
 # SVG version of gpar()
-assign("SVGParList",
-       # http://www.w3.org/TR/SVG11/shapes.html#RectElement
-       c(# graphical event attributes
-         'onfocusin', 'onfocusout', 'onactivate', 'onclick', 'onmousedown',
-         'onmouseup', 'onmouseover', 'onmousemove', 'onmouseout', 'onload',
-         # presentation attributes
-         'alignment-baseline', 'baseline-shift', 'clip', 'clip-path',
-         'clip-rule', 'color', 'color-interpolation',
-         'color-interpolation-filters', 'color-profile', 'color-rendering',
-         'cursor', 'direction', 'display', 'dominant-baseline',
-         'enable-background', 'fill', 'fill-opacity', 'fill-rule',
-         'filter', 'flood-color', 'flood-opacity', 'font-family', 'font-size',
-         'font-size-adjust', 'font-stretch', 'font-style', 'font-variant',
-         'font-weight', 'glyph-orientation-horizontal',
-         'glyph-orientation-vertical', 'image-rendering', 'kerning',
-         'letter-spacing', 'lighting-color', 'marker-end', 'marker-mid',
-         'marker-start', 'mask', 'opacity', 'overflow', 'pointer-events',
-         'shape-rendering', 'stop-color', 'stop-opacity', 'stroke',
-         'stroke-dasharray', 'stroke-dashoffset', 'stroke-linecap',
-         'stroke-linejoin', 'stroke-miterlimit', 'stroke-opacity',
-         'stroke-width', 'text-anchor', 'text-decoration', 'text-rendering',
-         'unicode-bidi', 'visibility', 'word-spacing', 'writing-mode'),
-       envir=.gridSVGEnv)
 
 # NOTE using SVG presentation attributes
 # RATHER THAN CSS style attribute
@@ -1659,14 +1748,14 @@ assign("SVGParList",
 # - presentation attributes have lower priority than CSS style
 #   SO this allows overriding by specifying CSS style later.
 #   Can also override with general style sheet later.
-svgStyleAttributes <- function(svgstyle) {
+svgStyleAttributes <- function(svgstyle, dev) {
     if (emptyStyle(svgstyle)) {
         list()
     } else {
         # Remove non-SVG style attributes (with warning)
         names <- names(svgstyle)
         svgnames <- names %in% get("SVGParList", envir=.gridSVGEnv)
-        if (any(!svgnames)) {
+        if (svgStrict(dev) && any(!svgnames)) {
             warning(paste("Removing non-SVG style attribute name(s):",
                           paste(names[!svgnames], collapse=", ")))
             svgstyle <- svgstyle[names[svgnames]]
@@ -1719,3 +1808,17 @@ alignmentBaseline <- function(vjust) {
               "baseline"))
 }
 
+splitOnNA <- function(x, y) {
+    # Check for non-finite (rather than just NA)
+    nas <- !is.finite(x) | !is.finite(y)
+    wnas <- which(nas)
+    N <- max(length(x), length(y))
+    start <- c(1, pmin(wnas + 1, N))
+    end <- c(pmax(wnas - 1, 1), N)
+    xylist <- mapply(function(s, e) {
+                         if (e > s) list(x=x[s:e], y=y[s:e])
+                     },
+                     start, end, SIMPLIFY=FALSE)
+    xylist[!sapply(xylist, is.null)]
+}
+
